var $page = {
    bindableVars: [
        'view',
        'context',
        'balaio',
        'balaioDefault',
        'balaiosList',
        'mucua',
        'mucuaDefault',
        'mucuasList',
        'session',
        'message',
    ],

    init: function () {
        this.loadBalaiosList();
        this.loadBalaioDefault();
        this.loadMucuaDefault();

        $t("body");
    },

    loadBalaioDefault: function () {
        this.load('GET', 'balaio/default', null, function (response) {
            $page.balaioDefault = response;
        });
    },
    loadBalaiosList: function () {
        this.load('GET', 'balaio', null, function (response) {
            $page.balaiosList = response;
        });
    },
    loadMucuaDefault: function () {
        this.load('GET', 'mucua/default', null, function (response) {
            $page.mucuaDefault = response;
        });
    },
    loadMucuasList: function () {
        this.load('GET', this.balaio.smid+'/mucua', null, function (response) {
            $page.mucuasList = response;
        });
    },
    loadSession: function(token) {
        this.load('GET', 'session?token='+token, null, function(response) {
            $page.session = response;
        });
    },
    postAuth: function(username, password) {
        var data = 'username='+username+'&password='+password;
        this.load('POST', 'auth', data, function (response) {
            $page.session = response;
            $page.msg("Mocambola autenticado!");
        });
    },
    load: function (method, endpoint, data, success, error) {
        const xhr = new XMLHttpRequest();
        xhr.onload = function () {
            if (xhr.status == 200) {
                success(JSON.parse(xhr.response));
            } else {
                // TODO global error handling
                $page.msg('Verifique nome e senha', 'danger');
            }
        };
        xhr.onerror = function() {
            // TODO global error handling
            console.log(xhr);
            if (error) {
                console.log(error);
                $page.msg(xhr.statusText, danger);
            }
        };
        xhr.open(method, API_URL+endpoint);
        xhr.setRequestHeader("Cache-Control", "no-cache");
        xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        if (this.session){
            xhr.setRequestHeader("token", this.session.token);
        }
        xhr.send(data);
    },

    msg: function (header, type = 'info', detail = null) {
        this.message = {
            type: type,
            header: header,
            detail: detail
        };
    },

    $privateScope: {},
    bindable: function (varName) {
        Object.defineProperty(this, varName, {
            get: function () {
                return $page.$privateScope[varName];
            },
            set: function (newValue) {
                oldValue = $page.$privateScope[varName];
                $page.$privateScope[varName] = newValue;
                $page.fireChangeEvent(varName, oldValue, newValue);
            }
        });
    },
    onChange: function (varName, callback) {
        if (!this.$privateScope.changeCallbacks) {
            this.$privateScope.changeCallbacks = {};
        }
        if (!this.$privateScope.changeCallbacks[varName]) {
            this.$privateScope.changeCallbacks[varName] = [];
        }
        this.$privateScope.changeCallbacks[varName].push(callback);
    },
    fireChangeEvent: function(varName, oldValue, newValue) {
        if (!this.$privateScope.changeCallbacks) {
            return;
        }
        if (!this.$privateScope.changeCallbacks[varName]) {
            return;
        }
        var event = {
            varName: varName,
            oldValue: oldValue,
            newValue: newValue
        }
        for (var i=0; i<this.$privateScope.changeCallbacks[varName].length; i++) {
            this.$privateScope.changeCallbacks[varName][i](event);
        }
    }
};

$(function () {
    for (var i = 0; i<$page.bindableVars.length; i++) {
        $page.bindable($page.bindableVars[i]);
    }
    $page.onChange('balaioDefault', function () {
        if (!$page.balaio) {
            $page.balaio = $page.balaioDefault;
        }
    });
    $page.onChange('mucuaDefault', function () {
        if (!$page.mucua) {
            $page.mucua = $page.mucuaDefault;
        }
    });
    $page.onChange('balaio', function () {
        $page.loadMucuasList();
    });
    $page.onChange('session', function() {
        $page.loadBalaiosList();
    })
});